<?php

###########################################################################
# All Calculator code is Copyright 2021 by Cyril MAGUIRE (https://bookmarks.ecyseo.net).

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or (at
## your option) any later version.

## This program is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
## or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
## for more details.

## You should have received a copy of the GNU General Public License
## along with this program as the file LICENSE.txt; if not, please see
## http:##www.gnu.org/licenses/old-licenses/gpl-2.0.txt.

## This programm includes works under other copyright notices and distributed
## according to the terms of the GNU General Public License or a compatible
## license, including:

## PHP

##   Zip class - Copyright (c) 2020 https://www.warriordudimanche.net (bisous copain :D)

#########################################################################
#                                                                       #
#  ████   ████  ████    ████  ██  ██ ████    ████  ██████  ████  █████  #
# ██  ██ ██  ██  ██    ██  ██ ██  ██  ██    ██  ██ █ ██ █ ██  ██ ██  ██ #
# ██     ██  ██  ██    ██     ██  ██  ██    ██  ██   ██   ██  ██ ██  ██ #
# ██     ██████  ██    ██     ██  ██  ██    ██████   ██   ██  ██ █████  #
# ██     ██  ██  ██    ██     ██  ██  ██    ██  ██   ██   ██  ██ ██  ██ #
# ██  ██ ██  ██  ██ ██ ██  ██ ██  ██  ██ ██ ██  ██   ██   ██  ██ ██  ██ #
#  ████  ██  ██ ██████  ████   ████  ██████ ██  ██  ████   ████  ██  ██ #
#                                                                       #
#########################################################################

namespace Calculator\Config {
    define('VERSION','1');
    $urlCalculatorSnippet = $_SERVER['SCRIPT_URI'];
    $snippet = 'javascript:(function(){window.open(\''.$urlCalculatorSnippet.'\',\'_blank\',\'menubar=no,height=520,width=500,toolbar=no,scrollbars=no,status=no,dialog=1\');})();';
}

namespace Calculator {

    define('PHP_VERSION_MIN','7.1.0');
    define('WRONG_PHP_VERSION','Wrong PHP version.');
    define('MIN_PHP_VERSION','Minimal version required : ');
    define('L_INFO_PHP_VERSION','php version');

    define('HTML','<!DOCTYPE html><html lang="fr"><head><meta charset="utf8"></head><body><h1>Erreur !</h1><div>');
    define('FOOTER','</div></body></html>');

    if(version_compare(PHP_VERSION, PHP_VERSION_MIN, '<')){
        header('Content-Type: text/html charset=UTF-8');
        echo HTML.WRONG_PHP_VERSION.'<br/>'.MIN_PHP_VERSION.PHP_VERSION_MIN.'<br/>'.L_INFO_PHP_VERSION.' : '.phpversion().FOOTER;
        exit();
    }

    interface TerminalExpressionInterface {
        public function __construct($value);
        public function render();
    }

    interface OperatorInterface {

        const IS_OPERATOR= true;
        const ISNOT_OPERATOR= false;

        const IS_PARENTHESIS = true;
        const ISNOT_PARENTHESIS = false;

        const IS_NOOP = true;
        const ISNOT_NOOP = false;


        public function __construct(
            TerminalExpressionInterface $TerminalExpression,
            bool $isOperator = false,
            bool $isParenthesis = false,
            bool $isNoOp = false
        );
        public function getValue();
        public function isOperator():bool;
        public function isNoOp():bool;
        public function isParenthesis():bool;
    }

    interface OperationInterface {
        public function __construct(OperatorInterface $Operator);
        public function operator():OperatorInterface;
        public function operate(Stack $stack);
        public function isLeftAssoc():bool;
        public function getPrecidence():int;
        public function getSymbols():string;
        public function isOpen():bool;
    }


    final class Stack {

        private $data = array();

        public function push($element) {
            $this->data[] = $element;
        }

        public function poke() {
            return end($this->data);
        }

        public function pop() {
            return array_pop($this->data);
        }
    }


    final class TerminalExpression implements TerminalExpressionInterface{

        private $value = '';
        private $availableOperations = array();

        public function __construct($value) {
            $this->value = $value;
        }

        private function getDeclaredOperations()
        {
            foreach (get_declared_classes() as $key => $class) {
                if (strpos($class, __NAMESPACE__ . '\Operations') !== false ) {
                    if (class_implements($class,true)) {
                        $this->availableOperations[(new $class(new Operator(
                                        new TerminalExpression(''),
                                        OperatorInterface::IS_OPERATOR,
                                        OperatorInterface::ISNOT_PARENTHESIS,
                                        OperatorInterface::ISNOT_NOOP
                                        )))->getSymbols()] = $class;
                    }
                }
            }
            return $this->availableOperations;
        }

        public function getAvailableOperations()
        {
            $this->getDeclaredOperations();
            return array_keys($this->availableOperations);
        }

        public function isOperationAvailable(string $value):?string
        {
            $symbols = $this->getAvailableOperations();

            foreach ($symbols as $key => $op) {
                if (strpos($op,$value) !== false) {
                    return $this->availableOperations[$symbols[$key]];
                }
            }
            return null;
        }

        public function factory($value) {
            if (is_object($value) && $value instanceof TerminalExpression) {
                return $value;
            } elseif (is_numeric($value)) {
                return new Number(new Operator(
                                    new TerminalExpression($value),
                                    OperatorInterface::ISNOT_OPERATOR,
                                    OperatorInterface::ISNOT_PARENTHESIS,
                                    OperatorInterface::ISNOT_NOOP)
                );
            } elseif ($operation = $this->isOperationAvailable($value)) {
                return new $operation(new Operator(
                                        new TerminalExpression($value),
                                        OperatorInterface::IS_OPERATOR,
                                        OperatorInterface::ISNOT_PARENTHESIS,
                                        OperatorInterface::ISNOT_NOOP
                                        )
                );
            } elseif (in_array($value, array('(', ')'))) {
                return new Parenthesis(new Operator(
                                        new TerminalExpression($value),
                                        OperatorInterface::ISNOT_OPERATOR,
                                        OperatorInterface::IS_PARENTHESIS,
                                        OperatorInterface::IS_NOOP
                                        )
                );
            } elseif (is_nan($value)) {
                $this->value = $value;
                return $this->render();
            }
            throw new \Exception('Undefined Value ' . $value);
        }

        public function render() {
            return $this->value;
        }
    }


    final class Operator implements OperatorInterface {
        private $isParenthesis = false;
        private $isOperator = true;
        private $isNoOp = false;
        private $value = '';
        private $TerminalExpression;

        public function __construct(
            TerminalExpressionInterface $TerminalExpression,
            bool $isOperator = false,
            bool $isParenthesis = false,
            bool $isNoOp = false
        ) {
            $this->TerminalExpression = $TerminalExpression;
            $this->value = $this->TerminalExpression->render();
            $this->isOperator = $isOperator;
            $this->isNoOp = $isNoOp;
            $this->isParenthesis = $isParenthesis;
        }

        public function getValue() {
            return $this->value;
        }

        public function getPrecidence():int {
            return $this->precidence;
        }

        public function isOperator():bool {
            return $this->isOperator;
        }

        public function isNoOp():bool {
            return $this->isNoOp;;
        }

        public function isParenthesis():bool {
            return $this->isParenthesis;
        }

    }


    final class Parenthesis implements OperationInterface {


        private $leftAssoc = false;
        private $Operator;
        private $precidence = 6;

        public function __construct(OperatorInterface $Operator) {
            $this->Operator = $Operator;
        }
        public function operator():OperatorInterface {
            return $this->Operator;
        }
        public function operate(Stack $stack) {
            return false;
        }
        public function isLeftAssoc():bool {
            return $this->leftAssoc;
        }
        public function getPrecidence():int {
            return $this->precidence;
        }

        public function getSymbols():string {
            return $this->Operator->getValue();
        }

        public function isOpen():bool {
            return $this->Operator->getValue() == '(';
        }
    }


    final class Number implements OperationInterface {

        private $value = array();
        private $leftAssoc = false;
        private $Operator;
        private $precidence = 0;

        public function __construct(OperatorInterface $Operator) {
            $this->Operator = $Operator;
            $this->value = $this->Operator->getValue();
        }
        public function operator():OperatorInterface {
            return $this->Operator;
        }
        public function operate(Stack $stack) {
            return $this->value;
        }
        public function isLeftAssoc():bool {
            return $this->leftAssoc;
        }
        public function getPrecidence():int {
            return $this->precidence;
        }
        public function getSymbols():string {
            return $this->value;
        }
        public function isOpen():bool {
            return false;
        }
    }


    final class Math {

        private $variables = array();

        public function evaluate($string) {
            $stack = $this->parse($string);
            return $this->run($stack);
        }

        public function parse($string) {
            $tokens = $this->tokenize($string);
            $output = new Stack();
            $operators = new Stack();
            foreach ($tokens as $token) {
                $token = $this->extractVariables($token);
                $expression = (new TerminalExpression($token))->factory($token);
                if ($expression->operator()->isOperator()) {
                    $this->parseOperator($expression, $output, $operators);
                } elseif ($expression->operator()->isParenthesis()) {
                    $this->parseParenthesis($expression, $output, $operators);
                } elseif (is_float($expression) && is_nan($expression)) {
                    return $this->render((new Stack())->push($expression));
                } else {
                    $output->push($expression);
                }
            }
            while (($op = $operators->pop())) {
                if ($op->operator()->isParenthesis()) {
                    throw new \RuntimeException('Mismatched Parenthesis');
                }
                $output->push($op);
            }
            return $output;
        }

        public function registerVariable($name, $value) {
            $this->variables[$name] = $value;
        }

        public function run(Stack $stack) {
            while (($operator = $stack->pop()) && $operator->operator()->isOperator()) {
                $value = $operator->operate($stack);
                if (!is_null($value)) {
                    $stack->push((new TerminalExpression($value))->factory($value));
                }
            }
            return $operator ? $operator->operator()->getValue() : $this->render($stack);
        }

        private function extractVariables($token) {
            if ($token[0] == '$') {
                $key = substr($token, 1);
                return isset($this->variables[$key]) ? $this->variables[$key] : 0;
            }
            return $token;
        }

        private function render(Stack $stack) {
            $output = '';
            while (($el = $stack->pop())) {
                $output .= $el->render();
            }
            if ($output) {
                return $output;
            }
            throw new \RuntimeException('Could not render output');
        }

        private function parseParenthesis(OperationInterface $expression, Stack $output, Stack $operators) {
            if ($expression->isOpen()) {
                $operators->push($expression);
            } else {
                $clean = false;
                while (($end = $operators->pop())) {
                    if ($end->operator()->isParenthesis()) {
                        $clean = true;
                        break;
                    } else {
                        $output->push($end);
                    }
                }
                if (!$clean) {
                    throw new \RuntimeException('Mismatched Parenthesis');
                }
            }
        }

        private function parseOperator(OperationInterface $expression, Stack $output, Stack $operators) {
            $end = $operators->poke();
            if (!$end) {
                $operators->push($expression);
            } elseif ($end->operator()->isOperator()) {
                do {
                    if ($expression->isLeftAssoc() && $expression->getPrecidence() <= $end->getPrecidence()) {
                        $output->push($operators->pop());
                    } elseif (!$expression->isLeftAssoc() && $expression->getPrecidence() < $end->getPrecidence()) {
                        $output->push($operators->pop());
                    } else {
                        break;
                    }
                } while (($end = $operators->poke()) && $end->operator()->isOperator());
                $operators->push($expression);
            } else {
                $operators->push($expression);
            }
        }

        private function sign( $number ) {
            return ( $number > 0 ) ? 1 : ( ( $number < 0 ) ? -1 : 0 );
        }

        private function tokenize($string) {
            $parts = preg_split('((\-?\d+\.?\d*|\+|-|\(|\)|\*|/)|\s+)', $string, null, PREG_SPLIT_NO_EMPTY | PREG_SPLIT_DELIM_CAPTURE);
            $parts = array_map('trim', $parts);
            $token = array();
            $type = array();
            foreach ($parts as $key => $value) {
                // For negative numbers
                if ($this->sign($value) == -1) {
                    if (isset($parts[$key-1]) && is_numeric($parts[$key-1])) {
                        $token[] = '+';
                        $token[] = $value;
                    } else {
                        $token[] = $value;
                    }
                } else {
                    // if a number
                    if (isset($parts[$key-1]) &&
                        is_numeric($parts[$key-1]) &&
                        !is_numeric($value) &&
                        !in_array($value, array('*','x','X','+','-','−','/','÷','%','^','mod'))
                    ) {
                        if (isset($parts[$key+1]) && $parts[$key+1] != ')') {
                            $token[] = '*';
                            $token[] = $value;
                        } else {
                            $token[] = $value;
                        }
                    } else {
                        $token[] = $value;
                    }
                }
            }
            return $token;
        }

    }

}

#########################################################################
#                                                                       #
#  ████  █████  ██████ █████   ████  ██████  ████   ████  █   ██  ████  #
# ██  ██ ██  ██ ██     ██  ██ ██  ██ █ ██ █   ██   ██  ██ ██  ██ ██  ██ #
# ██  ██ ██  ██ ██     ██  ██ ██  ██   ██     ██   ██  ██ ███ ██  ██    #
# ██  ██ █████  █████  █████  ██████   ██     ██   ██  ██ ██████   ██   #
# ██  ██ ██     ██     ██  ██ ██  ██   ██     ██   ██  ██ ██ ███    ██  #
# ██  ██ ██     ██     ██  ██ ██  ██   ██     ██   ██  ██ ██  ██ ██  ██ #
#  ████  ██     ██████ ██  ██ ██  ██  ████   ████   ████  ██   █  ████  #
#                                                                       #
#########################################################################

namespace Calculator\Operations {

    ///////////////////////////////////////////////////////
    //     _________________________________________     //
    //     |          |             |               |    //
    //     | Operator |  Precedence | Associativity |    //
    //     |__________|_____________|_______________|    //
    //     |          |             |               |    //
    //     |    mod   |      4      |     Right     |    //
    //     |    √     |      4      |     Right     |    //
    //     |    ^     |      4      |     Right     |    //
    //     |    %     |      3      |     Left      |    //
    //     |    ×     |      3      |     Left      |    //
    //     |    ÷     |      3      |     Left      |    //
    //     |    +     |      2      |     Left      |    //
    //     |    −     |      2      |     Left      |    //
    //     |__________|_____________|_______________|    //
    //                                                   //
    ///////////////////////////////////////////////////////

    final class Addition implements \Calculator\OperationInterface  {

        private $symbols = array('+');
        private $value = '';
        private $Operator;
        private $precidence = 2;
        private $leftAssoc = true;

        public function __construct(\Calculator\OperatorInterface $Operator) {
            $this->Operator = $Operator;
            $this->value = $this->Operator->getValue();
        }

        public function operator():\Calculator\OperatorInterface {
            return $this->Operator;
        }

        public function operate(\Calculator\Stack $stack) {
            return $stack->pop()->operate($stack) + $stack->pop()->operate($stack);
        }

        public function isLeftAssoc():bool {
            return $this->leftAssoc;
        }

        public function getPrecidence():int {
            return $this->precidence;
        }

        public function getSymbols():string {
            return implode('',$this->symbols);
        }

        public function isOpen():bool {
            return false;
        }

    }

    final class Subtraction implements \Calculator\OperationInterface {

        private $symbols = array('-','−');
        private $value = '';
        private $Operator;
        private $precidence = 2;
        private $leftAssoc = true;

        public function __construct(\Calculator\OperatorInterface $Operator) {
            $this->Operator = $Operator;
            $this->value = $this->Operator->getValue();
        }

        public function operator():\Calculator\OperatorInterface {
            return $this->Operator;
        }

        public function isLeftAssoc():bool {
            return $this->leftAssoc;
        }

        public function operate(\Calculator\Stack $stack) {
            $left = $stack->pop()->operate($stack);
            $right = $stack->pop();
            $right = ($right ? $right->operate($stack) : 0);
            return $right - $left;
        }

        public function getPrecidence():int {
            return $this->precidence;
        }

        public function getSymbols():string {
            return implode('',$this->symbols);
        }

        public function isOpen():bool {
            return false;
        }

    }

    final class Multiplication implements \Calculator\OperationInterface  {

        private $symbols = array('*','x','X');
        private $value = '';
        private $Operator;
        private $precidence = 3;
        private $leftAssoc = true;

        public function __construct(\Calculator\OperatorInterface $Operator) {
            $this->Operator = $Operator;
            $this->value = $this->Operator->getValue();
        }

        public function operator():\Calculator\OperatorInterface {
            return $this->Operator;
        }

        public function isLeftAssoc():bool {
            return $this->leftAssoc;
        }

        public function operate(\Calculator\Stack $stack) {
            return $stack->pop()->operate($stack) * $stack->pop()->operate($stack);
        }

        public function getPrecidence():int {
            return $this->precidence;
        }

        public function getSymbols():string {
            return implode('',$this->symbols);
        }

        public function isOpen():bool {
            return false;
        }

    }

    final class Division implements \Calculator\OperationInterface  {

        private $symbols = array('/','÷');
        private $value = '';
        private $Operator;
        private $precidence = 3;
        private $leftAssoc = true;

        public function __construct(\Calculator\OperatorInterface $Operator) {
            $this->Operator = $Operator;
            $this->value = $this->Operator->getValue();
        }

        public function operator():\Calculator\OperatorInterface {
            return $this->Operator;
        }

        public function operate(\Calculator\Stack $stack) {
            $left = $stack->pop()->operate($stack);
            $right = $stack->pop()->operate($stack);
            if ($left == 0) {
                try {
                    throw new \Exception('Division by zero not allowed');;
                } catch(\Exception $e) {
                    echo 'Error : '. $e->getMessage();
                }
            }
            return $right / $left;
        }

        public function isLeftAssoc():bool {
            return $this->leftAssoc;
        }

        public function getPrecidence():int {
            return $this->precidence;
        }

        public function getSymbols():string {
            return implode('',$this->symbols);
        }

        public function isOpen():bool {
            return false;
        }

    }

    final class Percentage implements \Calculator\OperationInterface  {

        private $symbols = array('%');
        private $value = '';
        private $Operator;
        private $precidence = 3;
        private $leftAssoc = true;

        public function __construct(\Calculator\OperatorInterface $Operator) {
            $this->Operator = $Operator;
            $this->value = $this->Operator->getValue();
        }

        public function operator():\Calculator\OperatorInterface {
            return $this->Operator;
        }

        public function operate(\Calculator\Stack $stack) {
            $left = $stack->pop()->operate($stack);
            $leftleft = clone $stack;
            $leftleft = $leftleft->pop();

            if ($leftleft !== null && ($leftleft instanceof \Calculator\Number) === true) {
                $leftleft = $leftleft->getSymbols();
                return ($left * $leftleft) / 100;
            }
            return ($left / 100);

        }

        public function isLeftAssoc():bool {
            return $this->leftAssoc;
        }

        public function getPrecidence():int {
            return $this->precidence;
        }

        public function getSymbols():string {
            return implode('',$this->symbols);
        }

        public function isOpen():bool {
            return false;
        }

    }

    final class Exponent implements \Calculator\OperationInterface  {

        private $symbols = array('^');
        private $value = '';
        private $Operator;
        private $precidence = 4;
        private $leftAssoc = false;

        public function __construct(\Calculator\OperatorInterface $Operator) {
            $this->Operator = $Operator;
            $this->value = $this->Operator->getValue();
        }

        public function operator():\Calculator\OperatorInterface {
            return $this->Operator;
        }

        public function operate(\Calculator\Stack $stack) {
            $right = $stack->pop()->operate($stack);
            if (($left = $stack->pop()) !== null) {
                $left = $left->operate($stack);
            } else {
                return 0;
            }
            return $left ** $right;
        }

        public function isLeftAssoc():bool {
            return $this->leftAssoc;
        }

        public function getPrecidence():int {
            return $this->precidence;
        }

        public function getSymbols():string {
            return implode('',$this->symbols);
        }

        public function isOpen():bool {
            return false;
        }

    }

    final class SquareRoot implements \Calculator\OperationInterface  {

        private $symbols = array('sqrt','√');
        private $value = '';
        private $Operator;
        private $precidence = 4;
        private $leftAssoc = false;

        public function __construct(\Calculator\OperatorInterface $Operator) {
            $this->Operator = $Operator;
            $this->value = $this->Operator->getValue();
        }

        public function operator():\Calculator\OperatorInterface {
            return $this->Operator;
        }

        public function operate(\Calculator\Stack $stack) {
            $right = $stack->pop()->operate($stack);
            return sqrt($right);
        }

        public function isLeftAssoc():bool {
            return $this->leftAssoc;
        }

        public function getPrecidence():int {
            return $this->precidence;
        }

        public function getSymbols():string {
            return implode('',$this->symbols);
        }

        public function isOpen():bool {
            return false;
        }

    }

    final class Modulo implements \Calculator\OperationInterface  {

        private $symbols = array('mod');
        private $value = '';
        private $Operator;
        private $precidence = 4;
        private $leftAssoc = false;

        public function __construct(\Calculator\OperatorInterface $Operator) {
            $this->Operator = $Operator;
            $this->value = $this->Operator->getValue();
        }

        public function operator():\Calculator\OperatorInterface {
            return $this->Operator;
        }

        public function operate(\Calculator\Stack $stack) {

            $right = $stack->pop()->operate($stack);
            if (($left = $stack->pop()) !== null) {
                $left = $left->operate($stack);
            } else {
                return 0;
            }
            return $left % $right;
        }

        public function isLeftAssoc():bool {
            return $this->leftAssoc;
        }

        public function getPrecidence():int {
            return $this->precidence;
        }

        public function getSymbols():string {
            return implode('',$this->symbols);
        }

        public function isOpen():bool {
            return false;
        }

    }
}

######################################
#                                    #
# ██████ ██████  ████  ██████  ████  #
# █ ██ █ ██     ██  ██ █ ██ █ ██  ██ #
#   ██   ██      ██      ██    ██    #
#   ██   █████    ██     ██     ██   #
#   ██   ██        ██    ██      ██  #
#   ██   ██     ██  ██   ██   ██  ██ #
#  ████  ██████  ████   ████   ████  #
#                                    #
######################################
namespace Calculator\Tests\Operations {

    interface TestInterface {
        public function isEqual(string $expression, $result, $expected);
    }

    final class OperationTest implements TestInterface {
        public function isEqual(string $expression, $result, $expected)
        {
            $box = 'style="padding:10px;margin:10px 0;max-width:50%;';
            $Ok = 'color:green;background:#b5c4a8;border:1px solid green;"';
            $Ko = 'color:white;background:#ff2a2a;border:1px solid red;"';
            $s = ' style="color:black;"';


            eval($expected.';');

            $test = '<p><--------- TEST BEGIN ---------><br/>
            Expression : "'. $expression .'"<br/>
            Expected : "'. $expected .'"<br/>
            Result : "'. $result .'"<br/></p>';
            $testok = $test .'<p '.$box . $Ok .'>Test OK : "<i'. $s .'>'. $expression .'</i>" is equal to "<i'. $s .'>'. $expected .'</i>"</p>
            <p><br/><--------- TEST END ---------><br/><br/></p>';
            if ($result === $expected) {
                echo $testok;
            } elseif (is_nan($result) && is_nan($expected)) {
                echo $testok;
            } else {
                echo $test .'<p '.$box . $Ko .'>Test KO : "<i'. $s .'>'. $expression .'</i>" is not equal to "<i'. $s .'>'. $expected .'</i>"</p>';
                echo '<pre style="border: 1px solid #e3af43; background-color: #f8edd5; padding: 10px;color:#111;white-space: pre;white-space: pre-wrap;word-wrap: break-word;">Fichier '.__FILE__.', LIGNE '.__LINE__."<br/><strong>result :</strong>  "; var_dump($result);echo('</pre>');
                echo '<pre style="border: 1px solid #e3af43; background-color: #f8edd5; padding: 10px;color:#111;white-space: pre;white-space: pre-wrap;word-wrap: break-word;">Fichier '.__FILE__.', LIGNE '.__LINE__."<br/><strong>expected :</strong>  "; var_dump($expected);echo('</pre>');
                echo '<p><br/><--------- TEST END ---------><br/><br/></p>';
            }
        }
    }


}

#############################################
#                                           #
#  ████  ████    ████  █████   ████  ████   #
# ██  ██  ██    ██  ██ ██  ██ ██  ██  ██    #
# ██      ██    ██  ██ ██  ██ ██  ██  ██    #
# ██ ███  ██    ██  ██ █████  ██████  ██    #
# ██  ██  ██    ██  ██ ██  ██ ██  ██  ██    #
# ██  ██  ██ ██ ██  ██ ██  ██ ██  ██  ██ ██ #
#  ████  ██████  ████  █████  ██  ██ ██████ #
#                                           #
#############################################
namespace {

if (isset($_GET['download'])){
    $file='Calculator_v'.VERSION.'.zip';
    if (is_file($file)){
        header("Location: ".$file);
        exit;
    }
    $zip=new \Calculator\Vendor\zip();
    $files=glob('*.*');
    $files=array_reverse(array_reverse($files));
    # delete zip file from list
    foreach ($files as $key => $value) {
        if (stripos($value,'.zip')===false){
            $f[$key]=$value;
        }
    }
    $files=$f;
    $zip->setSource('.');
    $zip->setFilesList($files);
    $zip->setDestination($file);
    $zip->create();
    $zip->download();
    exit();
}

$test = new Calculator\Tests\Operations\OperationTest();
$math = new Calculator\Math();

$result = '';
$op = '';
$history = array();
if (isset($_POST['output'])) {

    $op = strip_tags($_POST['output']);

    if (isset($_POST['history'])) {
        $history = json_decode(base64_decode($_POST['history']), true);
    }

    if ($op != '') {
        $result = $math->evaluate($op);
        $history[] = array(
            'op' => $op,
            'result' => $result
        );
    }

}
?><!Doctype html>
<html>
   <head>
      <style>
        html {background-color:#454545;}
        table {border-collapse:collapse;cursor:pointer;}
        button.information,input[type=submit].information, input[type=button].information,.btn.information{background-color:#454545}
        td {background-color:#383838;padding:5px;cursor:pointer;width:71px;border:none;vertical-align:middle;}
        td p {color:white; padding:5px;font-size:12px;text-align:center;}
        td a {color:white;}
        td input {width:100%;border:1px solid #454545;border-radius:5px;cursor:pointer;padding:10px;color:white;}
        td :not(input[type=text]) {background-color:#454545;}
        td input[type=text] {background-color:#404040;border:none;}
        td input[type=submit] {background-color:#6db442;border:none;}
        td.erase input {background-color:#ec5f66;border:none;}
        td ul {list-style-type: none; color:white;}
        td ul {height:130px;min-height:130px;max-height:130px;overflow-y:auto;padding:10px}

        /* TESTS */
        p.test {padding:10px;margin:10px 0;max-width:50%;}
        p.test.ok {color:green;background:#b5c4a8;border:1px solid green;}
        p.test.ko {color:white;background:#ff2a2a;border:1px solid red;}
        i {color:black;}
      </style>
   </head>
   <body>
      <form action="" method="post" id="calculator">
          <table border="1">
             <tr>
                <td colspan="5"><input type="text" id="output<?php echo (!empty($result) ? 'send' : '');?>" name="output" value="<?php echo $result;?>"/></td>
                <td class="erase"><input type="button" onclick="effacer()" value="c" title="Erase"/></td>
             </tr>
             <tr>
                <td><input type="button" onclick="afficher('7')" value="7" title=" Seven [7]"/></td>
                <td><input type="button" onclick="afficher('8')" value="8" title="Eight [8]"/></td>
                <td><input type="button" onclick="afficher('9')" value="9" title="Nine [9]"/></td>
                <td><input type="button" onclick="afficher('/')" value="/" title="Division [÷]"/></td>
                <td><input type="button" onclick="afficher('^2')" value="x2" title="Square [^2]"/></td>
                <td><input type="button" onclick="afficher('√')" value="√" title="Square root [sqrt]"/></td>
             </tr>
             <tr>
                <td><input type="button" onclick="afficher('4')" value="4" title="Four [4]"/></td>
                <td><input type="button" onclick="afficher('5')" value="5" title="Five [5]"/></td>
                <td><input type="button" onclick="afficher('6')" value="6" title="Six [6]"/></td>
                <td><input type="button" onclick="afficher('*')" value="*" title="Multiplication [x]"/></td>
                <td><input type="button" onclick="afficher('^')" value="^" title="Exponent [^]" /></td>
                <td><input type="button" onclick="afficher('%')" value="%" title="Percentage [%]"/></td>
             </tr>
             <tr>
                <td><input type="button" onclick="afficher('1')" value="1" title="One [1]"/></td>
                <td><input type="button" onclick="afficher('2')" value="2" title="Two [2]"/></td>
                <td><input type="button" onclick="afficher('3')" value="3" title="Three [3]"/></td>
                <td><input type="button" onclick="afficher('-')" value="-" title="Substract [−]"/></td>
                <td><input type="button" onclick="afficher('(')" value="(" title="Start grouping [(]" /></td>
                <td><input type="button" onclick="afficher(')')" value=")" title="End grouping [)]" /></td>
             </tr>
             <tr>
                <td><input type="button" onclick="afficher('0')" value="0" title="Zero [0]"/></td>
                <td><input type="button" onclick="afficher('.')" value="." title="Dot [.]"/></td>
                <td><input type="button" onclick="afficher('mod(')" value="mod" title="Modulo [mod]"/></td>
                <td><input type="button" onclick="afficher('+')" value="+" title="Add [+]"/></td>
                <td colspan="2"><input type="submit" value="=" title="Calculate result"/></td>
             </tr>
             <?php if (!empty($history) && isset($_POST['output'])): ?>

             <tr>
                <td colspan="6">
                    <input type="hidden" name="history" value="<?php echo base64_encode(json_encode($history));?>">
                    <ul>
                     <?php foreach ($history as $key => $calcul) {
                         echo '<li onclick="effacer();afficher(\''.$calcul['op'].'\');">'. $calcul['op'] .' = '. $calcul['result'] .'</li>'."\n";
                     } ?>

                    </ul>
                </td>
            </tr>
            <?php else: ?>

             <tr>
                <td colspan="6">
                    <ul>
                        <li></li>
                    </ul>
                </td>
            </tr>
             <?php endif; ?>

             <tr>
                 <td colspan="6"><p><a href="<?php echo $snippet;?>" onclick="alertBookmarklet();return false;">Calculator</a> made with <img src="Heart_corazón.svg" width="15" height="15"/> by <a href="https://bookmarks.ecyseo.net">Jerry Wham</a> <a href="?download"><img src="cloud-download-alt.svg" title="Download version <?php echo VERSION;?>" width="15" height="15"/></a></p></td>
             </tr>

          </table>
      </form>

      <script>
        // Polyfill for trim()
        if (!String.prototype.trim) {
          String.prototype.trim = function () {
            return this.replace(/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g, '');
          };
        }
        function alertBookmarklet() {
            alert('Drag this link to your bookmarks toolbar, or right-click it and choose Bookmark This Link...');
            return false;
        }
        // We submit form on pressing '=' key
        document.addEventListener('keydown', (event) => {
          const nomTouche = event.key;

          if (nomTouche == '=' && verif() === true) {

            document.getElementById('calculator').submit();
          }
        }, false);
        //Display value
        function afficher(val)
        {
            const numbers = ['0','1','2','3','4','5','6','7','8','9','.'];
            var textareaSend = document.getElementById('outputsend');
            if (textareaSend !== null) {
                if (numbers.includes(val) == true ) {
                    textareaSend.value = '';
                } else {
                    textareaSend.value += ' ';
                    textareaSend.value = textareaSend.value.trim();
                }
                textareaSend.setAttribute('id','output');
                var textarea = textareaSend;
            } else {
                var textarea = document.getElementById('output');
            }
            var text = textarea.value;

            if (val != '=') {
                textarea.value = text.substring(0, textarea.selectionStart) + val +
                text.substring(textarea.selectionStart);

                textarea.focus();
            } else {
                document.getElementById('calculator').submit();
            }
        }
        //Erase value
        function effacer()
        {
            var textareaSend = document.getElementById('outputsend');
            if (textareaSend !== null) {
                textareaSend.value = '';
            } else {
                document.getElementById('output').value = '';
            }
        }
        //We check if value is entered before submit form
        var form = document.getElementById('calculator');
        form.addEventListener("submit", function (event) {
            event.preventDefault();
            verif();
            return false;
        });
        function verif() {
            if (document.getElementById('output').value != '') {
                document.getElementById('calculator').submit();
            } else {
                return false;
            }
        }
      </script>
   </body>
</html>

<?php
}

########################
#                      #
# ██████  ████  █████  #
# ██  ██   ██   ██  ██ #
#    ██    ██   ██  ██ #
#   ██     ██   █████  #
#  ██      ██   ██     #
# ██  ██   ██   ██     #
# ██████  ████  ██     #
#                      #
########################
namespace Calculator\Vendor{

    class zip{
        private $version="1.0";

        private $zipfile='';
        private $destination='';
        private $source='';
        private $fileslist=[];

        public function __construct(){

        }

        ###########################
        # METHODS #################
        ###########################

        # create
        ###########################
        public function create($source=null,$destination=null){
            if (
                  empty($source)
                &&empty($destination)
                &&empty($this->source)
                &&empty($this->destination)
                &&(empty($this->fileslist)||!is_array($this->fileslist))
            ){
                return false;
            }

            $source         =empty($source)?$this->source:$source;
            $source         =$this->addSlash($source);
            $destination    =empty($destination)?$this->destination:$destination;
            $this->fileslist=empty($this->fileslist)?$this->recursiveDirContent($source):$this->fileslist;

            $zip = new \ZipArchive();
            if (is_file($this->destination)){
                unlink($this->destination);
            }
            if (!$zip->open($this->destination, \ZIPARCHIVE::CREATE)) {
                throw new Exception("Error creating zipArchive", 1);
                return false;
            }

            foreach ($this->fileslist as $file) {
                $file=str_replace('./', '', $file);
                if (
                    $file!=$source
                    &&is_file($file)
                ){
                    $zip->addFile($file);
                }elseif (
                    $file!=$source
                    &&is_dir($file)
                ){
                    $zip->addEmptyDir($file);
                }
            }

            $this->zipfile=$destination;
        }
        private function addSlash($string){
            if (substr($string,strlen($string)-1,1)!='/'&&!empty($string)){return $string.'/';}else{return $string;}
        }
        private function recursiveDirContent($root=null,$filter=null){
            if (empty($root)){return false;}
            if (!is_dir($root)){return array($root);}
            $iter = new \RecursiveIteratorIterator(
                new \RecursiveDirectoryIterator($root, \RecursiveDirectoryIterator::SKIP_DOTS),
                \RecursiveIteratorIterator::SELF_FIRST,
                \RecursiveIteratorIterator::CATCH_GET_CHILD # Ignore "Permission denied"
            );

            $paths = array($root);
            foreach ($iter as $path => $dir) {
                if (
                    ($filter && stripos($path,$filter)!==false)
                    || !$filter
                ){
                    $paths[] = $path;
                }
            }

            return $paths;
        }
        # unzip
        ###########################
        public function unzip($file,$destination){
            if (
                  empty($source)
                &&empty($destination)
                &&empty($this->source)
                &&empty($this->destination)
            ){
                return false;
            }
            $zip = new ZipArchive() ;
            if ($zip->open($file) !== TRUE) {return false;}
            $zip->extractTo($destination);
            $zip->close();
            return true;
        }

        # download
        ###########################
        public function download(){
            header("Location: ".$this->zipfile);
            exit;
        }

        # getters
        public function getVersion(){return $this->version;}
        public function getDestination(){return $this->destination;}
        public function getSource(){return $this->source;}
        public function getFilesList(){return $this->fileslist;}

        # setters
        public function setVersion($value=null){$this->version=$value;}
        public function setDestination($value=null){$this->destination=$value;}
        public function setSource($value=null){$this->source=$value;}
        public function setFilesList($value=null){$this->fileslist=$value;}
    }
}
